@@ -19,6 +19,7 @@
 #' This will happen also if \code{verbose} is TRUE. Set both \code{verbose} and \code{precalculate} to FALSE to omit calculations.
 #' @param colorize if TRUE (default) then \code{WARNINGS}, \code{ERRORS} and \code{NOTES} are colorized. Will work only in the R console.
 #' @param model_info a named list (\code{package}, \code{version}, \code{type}) containg information about model. If \code{NULL}, \code{DALEX} will seek for information on it's own.
+#' @param type type of a model, either \code{classification} or \code{regression}. If not specified then \code{type} will be extracted from \code{model_info}.
 #'
 #' @return An object of the class \code{explainer}.
 #'
@@ -38,6 +39,7 @@
 #' \item \code{model_info} named list contating basic information about model, like package, version of package and type.
 #' }
 #'
+#' @references Explanatory Model Analysis. Explore, Explain and Examine Predictive Models. \url{https://pbiecek.github.io/ema/}
 #' @rdname explain
 #' @export
 #' @importFrom stats predict
@@ -64,14 +66,14 @@
 #' aps_lm <- explain(aps_lm_model4, data = apartments, label = "model_4v", y = apartments$m2.price,
 #'                                    predict_function = predict)
 #' # set model_info
-#' model_info <- list(package = "stats", ver = "3.6.1", type = "regression")
+#' model_info <- list(package = "stats", ver = "3.6.2", type = "regression")
 #' aps_lm_model4 <- lm(m2.price ~., data = apartments)
 #' aps_lm_explainer4 <- explain(aps_lm_model4, data = apartments, label = "model_4v",
 #'                              model_info = model_info)
 #'
 #' \dontrun{
 #' # set model_info
-#' model_info <- list(package = "stats", ver = "3.6.1", type = "regression")
+#' model_info <- list(package = "stats", ver = "3.6.2", type = "regression")
 #' aps_lm_model4 <- lm(m2.price ~., data = apartments)
 #' aps_lm_explainer4 <- explain(aps_lm_model4, data = apartments, label = "model_4v",
 #'                              model_info = model_info)
@@ -80,19 +82,17 @@
 #'                              weights = as.numeric(apartments$construction.year > 2000))
 #'
 #' # more complex model
-#' library("randomForest")
-#' aps_rf_model4 <- randomForest(m2.price ~., data = apartments)
-#' aps_rf_explainer4 <- explain(aps_rf_model4, data = apartments, label = "model_rf")
-#' aps_rf_explainer4
+#' library("ranger")
+#' aps_ranger_model4 <- ranger(m2.price ~., data = apartments, num.trees = 50)
+#' aps_ranger_explainer4 <- explain(aps_ranger_model4, data = apartments, label = "model_ranger")
+#' aps_ranger_explainer4
 #'  }
 #'
 
-#' @export
-#' @rdname explain
 explain.default <- function(model, data = NULL, y = NULL, predict_function = NULL,
                             residual_function = NULL, weights = NULL, ...,
                             label = NULL, verbose = TRUE, precalculate = TRUE,
-                            colorize = TRUE, model_info = NULL) {
+                            colorize = TRUE, model_info = NULL, type = NULL) {
 
   verbose_cat("Preparation of a new explainer is initiated\n", verbose = verbose)
 
@@ -110,22 +110,27 @@ explain.default <- function(model, data = NULL, y = NULL, predict_function = NUL
     label <- tail(class(model), 1)
     verbose_cat("  -> model label       : ", label, " (",color_codes$yellow_start,"default",color_codes$yellow_end,")\n", verbose = verbose)
   } else {
+    if (!is.character(label)) {
+      label <- substr(as.character(label), 1 , 15)
+      verbose_cat("  -> model label       : 'label' was not a string class object. Converted. (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
+    }
     verbose_cat("  -> model label       : ", label, "\n", verbose = verbose)
   }
 
   # REPORT: checks for data
   if (is.null(data)) {
-    # data not specified
-    # try to extract something
+    n <- 0
     possible_data <- try(model.frame(model), silent = TRUE)
-    if (class(possible_data) != "try-error") {
+    if (class(possible_data)[1] != "try-error") {
       data <- possible_data
-      verbose_cat("  -> data              : ", nrow(data), " rows ", ncol(data), " cols (\033[33mextracted from the model\033[39m)\n", verbose = verbose)
+      n <- nrow(data)
+      verbose_cat("  -> data              : ", n, " rows ", ncol(data), " cols", color_codes$yellow_start, "extracted from the model", color_codes$yellow_end, "\n", verbose = verbose)
     } else {
       verbose_cat("  -> no data avaliable! (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
     }
   } else {
-    verbose_cat("  -> data              : ", nrow(data), " rows ", ncol(data), " cols \n", verbose = verbose)
+    n <- nrow(data)
+    verbose_cat("  -> data              : ", n, " rows ", ncol(data), " cols \n", verbose = verbose)
   }
   # as for issue #15, if data is in the tibble format then needs to be translated to data.frame
   if ("tbl" %in% class(data)) {
@@ -133,17 +138,21 @@ explain.default <- function(model, data = NULL, y = NULL, predict_function = NUL
     verbose_cat("  -> data              :  tibbble converted into a data.frame \n", verbose = verbose)
   }
 
-  # REPORT: checks for y
+  # REPORT: checks for y present while data is NULL
   if (is.null(y)) {
     # y not specified
     verbose_cat("  -> target variable   :  not specified! (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
   } else {
+    if (is.null(data)) {
+      verbose_cat("  -> target variable   :  'y' present while data is NULL. (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
+
+    }
     if (is.data.frame(y)) {
       y <- unlist(y, use.names = FALSE)
       verbose_cat("  -> target variable   :  Argument 'y' was a data frame. Converted to a vector. (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
     }
     verbose_cat("  -> target variable   : ", length(y), " values \n", verbose = verbose)
-    if (length(y) != nrow(data)) {
+    if (length(y) != n) {
       verbose_cat("  -> target variable   :  length of 'y' is different than number of rows in 'data' (",color_codes$red_start,"WARNING",color_codes$red_end,") \n", verbose = verbose)
     }
     if ((is.factor(y) | is.character(y))) {
@@ -152,9 +161,11 @@ explain.default <- function(model, data = NULL, y = NULL, predict_function = NUL
       verbose_cat("  -> target variable   :  Otherwise I will not be able to calculate residuals or loss function.\n", verbose = verbose)
     }
 
-    if (!is.null(data) & is_y_in_data(data, y)) {
-      verbose_cat("  -> data              :  A column identical to the target variable `y` has been found in the `data`.  (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
-      verbose_cat("  -> data              :  It is highly recommended to pass `data` without the target variable column\n", verbose = verbose)
+    if (!is.null(data)) {
+      if (is_y_in_data(data, y)) {
+        verbose_cat("  -> data              :  A column identical to the target variable `y` has been found in the `data`.  (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
+        verbose_cat("  -> data              :  It is highly recommended to pass `data` without the target variable column\n", verbose = verbose)
+      }
     }
   }
 
@@ -163,16 +174,32 @@ explain.default <- function(model, data = NULL, y = NULL, predict_function = NUL
     # weights not specified
     # do nothing
   } else {
+    if (is.null(data)) {
+      verbose_cat("  -> sampling weights  :  'weights' present while data is NULL. (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
+    }
     if (is.data.frame(weights)) {
       weights <- unlist(weights, use.names = FALSE)
       verbose_cat("  -> sampling weights  :  Argument 'weights' was a data frame. Converted to a vector. (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
     }
     verbose_cat("  -> sampling weights  : ", length(weights), " values (",color_codes$yellow_start,"note that not all explanations handle weights",color_codes$yellow_end,")\n", verbose = verbose)
-    if (length(weights) != nrow(data)) {
+    if (length(weights) != n) {
       verbose_cat("  -> sampling weights  :  length of 'weights' is different than number of rows in 'data' (",color_codes$red_start,"WARNING",color_codes$red_end,") \n", verbose = verbose)
     }
   }
 
+  if (is.null(model_info)) {
+    # extract defaults
+    model_info <- model_info(model)
+    verbose_cat("  -> model_info        :  package", model_info$package[1], ", ver.", model_info$ver[1], ", task", model_info$type, "(", color_codes$yellow_start,"default",color_codes$yellow_end, ")", "\n", verbose = verbose)
+  } else {
+    verbose_cat("  -> model_info        :  package", model_info$package[1], ", ver.", model_info$ver[1], ", task", model_info$type, "\n", verbose = verbose)
+  }
+  # if type specified then it overwrite the type in model_info
+  if (!is.null(type)) {
+    model_info$type <- type
+    verbose_cat("  -> model_info        :  type set to ", type, "\n", verbose = verbose)
+  }
+
   # REPORT: checks for predict_function
   if (is.null(predict_function)) {
     # predict_function not specified
@@ -188,18 +215,24 @@ explain.default <- function(model, data = NULL, y = NULL, predict_function = NUL
       verbose_cat("  -> predict function  : ",matching_yhat[1]," will be used (",color_codes$yellow_start,"default",color_codes$yellow_end,")\n", verbose = verbose)
     }
   } else {
+    if (!"function" %in% class(predict_function)) {
+      verbose_cat("  -> predict function  : 'predict_function' is not a 'function' class object! (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
+
+    }
     verbose_cat("  -> predict function  : ", deparse(substitute(predict_function)), "\n", verbose = verbose)
   }
   # if data is specified then we may test predict_function
   y_hat <- NULL
   if (!is.null(data) & !is.null(predict_function) & (verbose | precalculate)) {
     y_hat <- try(predict_function(model, data), silent = TRUE)
-    if (class(y_hat) == "try-error") {
+    if (class(y_hat)[1] == "try-error") {
       y_hat <- NULL
       verbose_cat("  -> predicted values  :  the predict_function returns an error when executed (",color_codes$red_start,"WARNING",color_codes$red_end,") \n", verbose = verbose)
     } else {
       if ((is.factor(y_hat) | is.character(y_hat))) {
         verbose_cat("  -> predicted values  :  factor (",color_codes$red_start,"WARNING",color_codes$red_end,") with levels: ", paste(unique(y_hat), collapse = ", "), "\n", verbose = verbose)
+      } else if (!is.null(dim(y_hat))) {
+        verbose_cat("  -> predicted values  :  predict function returns multiple columns: ", ncol(y_hat), " (",color_codes$red_start,"WARNING",color_codes$red_end,") some of functionalities may not work \n", verbose = verbose)
       } else {
         verbose_cat("  -> predicted values  :  numerical, min = ", min(y_hat), ", mean = ", mean(y_hat), ", max = ", max(y_hat), " \n", verbose = verbose)
       }
@@ -217,13 +250,17 @@ explain.default <- function(model, data = NULL, y = NULL, predict_function = NUL
       verbose_cat("  -> residual function :  difference between y and yhat (",color_codes$yellow_start,"default",color_codes$yellow_end,")\n", verbose = verbose)
     }
   } else {
+    if (!"function" %in% class(residual_function)) {
+      verbose_cat("  -> residual function : 'residual_function' is not a 'function' class object! (",color_codes$red_start,"WARNING",color_codes$red_end,")\n", verbose = verbose)
+
+    }
     verbose_cat("  -> residual function : ", deparse(substitute(residual_function)), "\n", verbose = verbose)
   }
   # if data is specified then we may test residual_function
   residuals <- NULL
   if (!is.null(data) & !is.null(residual_function) & !is.null(y) & (verbose | precalculate)) {
     residuals <- try(residual_function(model, data, y), silent = TRUE)
-    if (class(residuals) == "try-error") {
+    if (class(residuals)[1] == "try-error") {
       residuals <- NULL
       verbose_cat("  -> residuals         :  the residual_function returns an error when executed (",color_codes$red_start,"WARNING",color_codes$red_end,") \n", verbose = verbose)
     } else {
@@ -234,14 +271,6 @@ explain.default <- function(model, data = NULL, y = NULL, predict_function = NUL
       }
     }
   }
-  if (is.null(model_info)) {
-    # extract defaults
-    model_info <- model_info(model)
-    verbose_cat("  -> model_info        :  package", model_info$package[1], ", ver.", model_info$ver[1], ", task", model_info$type, "(", color_codes$yellow_start,"default",color_codes$yellow_end, ")", "\n", verbose = verbose)
-  } else {
-    verbose_cat("  -> model_info        :  package", model_info$package[1], ", ver.", model_info$ver[1], ", task", model_info$type, "\n", verbose = verbose)
-  }
-
   # READY to create an explainer
 
   explainer <- list(model = model,
@@ -253,6 +282,7 @@ explain.default <- function(model, data = NULL, y = NULL, predict_function = NUL
                     class = class(model),
                     label = label,
                     model_info = model_info,
+                    residual_function = residual_function,
                     weights = weights)
   explainer <- c(explainer, list(...))
   class(explainer) <- "explainer"
